from pydantic import BaseModel, Field
from typing import List, Optional

from chiefpay.types.invoice import Address


class Wallet(BaseModel):
    id: str
    order_id: str = Field(alias="orderId")
    addresses: Optional[List[Address]] = Field(default=None)
    uuid: Optional[str] = Field(default=None)


from enum import Enum

BASE_URL = "https://api.chiefpay.org"

class Endpoints(Enum):
    invoices_history = '/v1/history/invoices'
    transactions_history = '/v1/history/transactions'
    rates = '/v1/rates'
    invoice = '/v1/invoice'
    wallet = '/v1/wallet'
    socket = '/socket.io'


from pydantic import BaseModel, Field

from chiefpay.types.wallet import Wallet


class Transaction(BaseModel):
    txid: str
    chain: str
    token: str
    value: str
    usd: str
    fee: str
    wallet: Wallet
    created_at: str = Field(alias='createdAt')
    block_created_at: str = Field(alias='blockCreatedAt')


from chiefpay import Client, SocketClient, AsyncClient, AsyncSocketClient
from chiefpay.constants import BASE_URL


class ChiefPayClient:
    def __init__(self, api_key: str, base_url: str = BASE_URL):
        self.rest = Client(api_key, base_url)
        self.socket = SocketClient(api_key, base_url)


class AsyncChiefPayClient:
    def __init__(self, api_key: str, base_url: str = BASE_URL):
        self.rest = AsyncClient(api_key, base_url)
        self.socket = AsyncSocketClient(api_key, base_url)



import socketio

from chiefpay.constants import BASE_URL
from chiefpay.exceptions import SocketError
from chiefpay.socket.base import BaseSocketClient
from chiefpay.types.notification import NotificationInvoice, NotificationTransaction


class SocketClient(BaseSocketClient):
    """
    Client for interacting with the payment system via WebSockets (synchronous).
    """
    def __init__(self, api_key: str, base_url: str = BASE_URL):
        super().__init__(api_key, base_url)
        self.sio = socketio.Client()
        self._setup_event_handlers()

    def _setup_event_handlers(self):
        @self.sio.event
        def connect():
            print("Connected to Socket.IO server")

        @self.sio.event
        def disconnect():
            print("Disconnected from Socket.IO server")

        @self.sio.event
        def rates(data):
            self.rates = data
            if self.on_rates:
                self.on_rates(data)

        @self.sio.event
        def notification(data: NotificationTransaction | NotificationInvoice):
            if self.on_notification:
                self.on_notification(data)

    def connect(self):
        """
        Connects to the Socket.IO server.

        Raises:
            SocketError: If the connection fails.
        """
        try:
            self.sio.connect(
                self.base_url, headers={"X-Api-Key": self.api_key}, socketio_path=self.PATH
            )
        except Exception as e:
            raise SocketError(f"Failed to connect to Socket.IO server: {e}")

    def disconnect(self):
        """
        Disconnects from the Socket.IO server.
        """
        self.sio.disconnect()

    def __enter__(self):
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.disconnect()


from typing import List
from pydantic import BaseModel, Field

from chiefpay.types.invoice import Invoice
from chiefpay.types.transaction import Transaction


class InvoicesHistory(BaseModel):
    invoices: List[Invoice]
    total_count: int = Field(alias='totalCount')


class TransactionsHistory(BaseModel):
    transactions: List[Transaction]
    total_count: int = Field(alias='totalCount')


class ChiefPayError(Exception):
    pass


class HTTPError(ChiefPayError):
    def __init__(self, status_code: int, message: str):
        self.status_code = status_code
        self.message = message
        super().__init__(f"HTTP Error: {status_code}: {message}")


class ManyRequestsError(ChiefPayError):
    pass


class APIError(ChiefPayError):
    pass


class SocketError(ChiefPayError):
    pass


from datetime import datetime


class Utils:
    @staticmethod
    def validate_date(date: str) -> bool:
        """
        Validates that the provided date
        """
        try:
            datetime.strptime(date, "%Y-%m-%dT%H:%M:%S.%fZ")
            return True
        except ValueError:
            raise ValueError(f"Invalid date format: {date}. Expected format is YYYY-MM-DDTHH:MM:SS.sssZ")



from setuptools import setup, find_packages

def requires():
    with open("requirements.txt", encoding="utf-16-le") as f:
        lst = f.read().splitlines()
        if lst and lst[0].startswith('\ufeff'):
            lst[0] = lst[0][1:]
        return lst

def readme():
  with open('README.md', 'r') as f:
    return f.read()

setup(
    name='chiefpay',
    version='1.0.1',
    packages=find_packages(),
    install_requires=requires(),
    author='nelsn',
    author_email='egor.larrr@gmail.com',
    description='ChiefPay Python SDK',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/ChiefPay/ChiefPay.py',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9',
)